
#!/usr/local/bin/perl
#
# This script attempts delivery of bsmtp'ed mail stored in file
# /var/spool/bsmtp/foo.com directly via smtp to mail.foo.com
# in response to a finger from that domain.
#
#       Written by Andy Rabagliati <andyr@wizzy.com> for
#       Rocky Mountain Internet <URL http://www.rmii.com>
#
#       Test from the command line with "deliver foo.com"
#
#####################################################################

require 'sock.pl';
require 'telnet.pl';
require 'newgetopt.pl';

# routine for clean shutdown on error
sub abort {
  &sock'close_all;
  die "ended unexpectedly, but shut down cleanly\n";
}
sub Lock {
    local($filename) = '/var/spool/bsmtp' . '/' . $_[0];
    return(0) unless -f $filename;      # no file to lock
    local($lockfile) = $filename . '.lock';
    # Try to get a lock on the file
    unlink($lockfile) if -M $lockfile > 0.2;    # delete old locks
    return(0)  if -f $lockfile;         # locked already
    link($filename, $lockfile);
    return (1);                         # we got it.
}

sub Unlock {
    local($lockfile) = '/var/spool/bsmtp' . '/' . $_[0] . '.lock';
    unlink($lockfile);
}

sub Retry {
    close(MAIL);
    $filename = @_;
    $top++;            # next number
    if ($filename =~ /(.+)\.([\d+])$/) {
        $sequence = $1;
        if ($sequence > $opt_retries) {
                $head = `head -30 $filename`;
                print $session "RSET\r\n";
                $_ = &telnet'read($session, $timeout);
                # open(BOUNCE, "|/bin/mail $from");
                open(BOUNCE, "|/bin/mail postmaster\@$domain $from");
                print BOUNCE <<EOM;
Precedence: junk
Subject: $status

An error occurred while delivering BSMTP mail to $to
in domain $domain after $opt_retries retries.

The status message was :-
$status
Part of the message follows:
$head
EOM
        } else {
          local($reject) = "$1.$top";
        }
    } else {
        local($reject) = "$filename.$top";
    }
    link($filename, $reject) && unlink ($filename);
}

&NGetOpt("prefix=s", "subdomain:i", "retries=i");

$opt_retries = 5 unless $opt_retries;

$hostname = shift(@ARGV);
@domain = split(/\./, $hostname);
while ($#domain > $opt_subdomain+1) {
        shift(@domain);
}
$domain = join('.', @domain);
if ($opt_prefix) {
    $gateway = $opt_prefix . '.' . $domain;
} else {
    $gateway =  $domain;
}
# print $domain, " ", $gateway, "\n";
$port = "smtp";
$timeout = 60;

opendir(SITE, '/var/spool/bsmtp');

# messages begin with q for first delivery, r for second, s for third.
# bounced messages stay around, beginning with z

@messages = grep((/^$domain$/ || /^$domain\.[\d+]$/), readdir(SITE));
closedir(SITE);

# print $gateway, "\n"; exit(0);

$session = &sock'open($gateway,$port) || die $!;

while (1) {
  $_ = &telnet'read($session, $timeout);
  &abort if &telnet'eof;
  print;
  last if m/^220 /;
}
print $session "HELO @HOSTNAME@\r\n";
$| = 1;

while (1) {
  $_ = &telnet'read($session, $timeout);
  &abort if &telnet'eof;
  print '? ', $_;
  last if m/^250/;
}

#       all files in directory

# print "opening /usr/spool/smail/out/$domain\n";

for $message (@messages) {
    $sequence = ($message =~ /[\d+]$/);
    if ($sequence > $top) {
        $top = $sequence;
    }
}

$top && print "Retry sequence numbers reach $top\n";

for $message (@messages) {
    $sequence = $1;
    $filename = '/var/spool/bsmtp' . '/' . $message;
    if (&Lock ($message)) {
        open(MAIL, $filename) ||
            die "Could not open $filename";
        print "Processing $message\n";
message:
        while (<MAIL>) {
            chop;
            if (/^MAIL FROM:\<(.*)\>$/) {
                $from = $1;
                print $session $_,"\r\n";
                # print $_,"\r\n";
                $status = &telnet'read($session, $timeout);
                # print $status;
                next message unless ($status =~ /^250/);
            } elsif (/^RCPT TO:\<(.*)\>$/) {
                $to = $1;
                print $session $_,"\r\n";
                # print $_,"\r\n";
                $status = &telnet'read($session, $timeout);
                # print $status;
                next message unless ($status =~ /^250/);
            } elsif (/^DATA/) {
                print $session $_,"\r\n";
                # print $_,"\r\n";
                $status = &telnet'read($session, $timeout);
                # print $status;
                next message unless ($status =~ /^354/);
                $_ = <MAIL>;
                while ($_ !~ /^\.$/)   # dot on its own
                {
                    chop;
                    print $session $_,"\r\n";
                    # print $_,"\r\n";
                    $_ = <MAIL>;
                }
                print $session $_,"\r\n";
                # print $_,"\r\n";
                $status = &telnet'read($session, $timeout);
                exit(1) if ( &telnet'timeout );
                if ($status =~ /^250/) {                # success
                    print $status;
                    sleep(2);                   # maybe this will help
                }
                next message unless ($status =~ /^250/);
            }
        }
        &Unlock ($message);
        close(MAIL);
        unlink ($filename);
    } else {
        print $message, "\n$filename locked\n";
    }
}
print $session "QUIT\r\n";
# print "QUIT\r\n";

# Get any exit messages
#
until (&telnet'eof) {
  print &telnet'read($session, $timeout);
}

&sock'close($session);
exit (0);


